/**
 * 平台将client端数据转换成为数据库交互对象(DB interact Object)的构造类的基准api定义
 */

"use strict";

var path = require('path'),
    lang = require(path.join(__dirname, "../utils/util-lang")),
    strUtil = require(path.join(__dirname, "../utils/util-string")),
    dateUtil = require(path.join(__dirname, "../utils/util-date")),
    DIO = require(path.join(__dirname, "../model/po-dio"));

var INSERT_TEMPLATE = "INSERT INTO %1(%2) VALUES(%3)",
    UPDATE_TEMPLATE = "UPDATE %1 SET %2 WHERE 1=1 %3",
    DELETE_TEMPLATE = "DELETE FROM %1 WHERE 1=1 %2";

function DIOGeneratorIntf(orms, sqlGetter) {
    this.orms = orms;
    this.sqlGetter = sqlGetter;
}

DIOGeneratorIntf.prototype = {
    constructor: DIOGeneratorIntf,
    /**
     * create the retrieve action of
     * @param domain 领域
     * @param module 模块
     * @param logicKey 业务逻辑对应的key
     * @param paramDic 参数字典
     * @returns undefined or DIO object instance
     */
    buildRetrieve: function (domain, module, logicKey, paramDic) {
        var self = this, sql, arr, dio;
        if (sql = self.sqlGetter.get(domain, module, logicKey)) {
            dio = new DIO();
            dio.sql = sql.sql;
            arr = dio.values = sql.in.sort.split(',');
            arr.forEach(function (item, index) {
                arr[index] = paramDic[item] || "";
            });
        }
        return dio;
    },
    /**
     * create the create action of sql
     * @param tbName
     * @param valueDic
     * @returns undefined or DIO object instance
     */
    buildCreate: function (tbName, valueDic) {
        var self = this, fields, dio, cols, vals, values, time;
        if (fields = self.orms[tbName]) {
            dio = new DIO();
            cols = [];
            vals = [];
            values = dio.values = [];
            fields.forEach(function (col) {
                if (valueDic.hasOwnProperty(col.name)) {
                    vals.push("?");
                    cols.push(col.name);
                    values.push(valueDic[col.name]);
                }
            });
            cols.push("createtime", "modifytime");
            values.push(time = dateUtil.dateToLong(new Date()), time);

            dio.sql = strUtil.formatStr(INSERT_TEMPLATE, tbName, cols.join(','), vals.join(','));
            dio.tbName = tbName;
            dio.action = "create";
        }
        return dio;
    },
    /**
     * create the update action of sql
     * @param tbName 表名
     * @param valueDic 待更新值的键值对
     * @param whereDic 待筛选的键值对
     * @returns {*}
     */
    buildUpdate: function (tbName, valueDic, whereDic) {
        var self = this, fields, dio, cols, wheres, values, colName;
        if (fields = self.orms[tbName]) {
            dio = new DIO();
            cols = [];
            wheres = [];
            values = dio.values = [];
            fields.forEach(function (col) {
                colName = col.name;
                if (valueDic.hasOwnProperty(colName)) {
                    cols.push(colName + "=?");
                    values.push(valueDic[colName]);
                } else if (whereDic.hasOwnProperty(colName)) {
                    wheres.push(colName + (col.symbol || "=") + "?");
                    values.push(whereDic[colName]);
                }
            });
            cols.push("modifytime");
            values.push(dateUtil.dateToLong(new Date()));

            dio.sql = strUtil.formatStr(UPDATE_TEMPLATE, tbName, cols.join(','),
                wheres.length ? ("and" + wheres.join('and')) : "");
            dio.tbName = tbName;
            dio.action = "update";
        }
        return dio;
    },
    /**
     * create the delete action of sql
     * @param tbName 表名
     * @param whereDic 待删除数据的筛选条件键值对
     */
    buildDelete: function (tbName, whereDic) {
        var self = this, fields, dio, wheres, values, colName;
        if (fields = self.orms[tbName]) {
            dio = new DIO();
            wheres = [];
            values = dio.values = [];
            fields.forEach(function (col) {
                colName = col.name;
                if (whereDic.hasOwnProperty(colName)) {
                    wheres.push(colName + (col.symbol || "=") + "?");
                    values.push(whereDic[colName]);
                }
            });

            dio.sql = strUtil.formatStr(DELETE_TEMPLATE, tbName, wheres.length ? ("and" + wheres.join('and')) : "");
            dio.tbName = tbName;
            dio.action = "delete";
        }
        return dio;
    },
    /**
     * create the recycle action of sql
     * @param tbName 表名
     * @param whereDic 待回收数据的筛选条件字符串
     * @returns {*}
     */
    buildRecycle: function (tbName, whereDic) {
        var self = this, fields, dio, wheres, values, colName;
        if (fields = self.orms[tbName]) {
            dio = new DIO();
            wheres = [];
            values = dio.values = [];
            fields.forEach(function (col) {
                colName = col.name;
                if (whereDic.hasOwnProperty(colName)) {
                    wheres.push(colName + (col.symbol || "=") + "?");
                    values.push(whereDic[colName]);
                }
            });
            dio.sql = strUtil.formatStr(UPDATE_TEMPLATE, tbName, "isdel=1,modifytime=" + dateUtil.dateToLong(new Date()),
                wheres.length ? ("and" + wheres.join('and')) : "");
            dio.tbName = tbName;
            dio.action = "recycle";
        }
        return dio;
    },
    /**
     * create the restore action of sql
     * @param tbName 表名
     * @param whereDic 待恢复数据的筛选条件字符串
     * @returns {*}
     */
    buildRestore: function (tbName, whereDic) {
        var self = this, fields, dio, wheres, values, colName;
        if (fields = self.orms[tbName]) {
            dio = new DIO();
            wheres = [];
            values = dio.values = [];
            fields.forEach(function (col) {
                colName = col.name;
                if (whereDic.hasOwnProperty(colName)) {
                    wheres.push(colName + (col.symbol || "=") + "?");
                    values.push(whereDic[colName]);
                }
            });
            dio.sql = strUtil.formatStr(UPDATE_TEMPLATE, tbName, "isdel=0,modifytime=" + dateUtil.dateToLong(new Date()),
                wheres.length ? ("and" + wheres.join('and')) : "");
            dio.tbName = tbName;
            dio.action = "restore";
        }
        return dio;
    }
};


module.exports = DIOGeneratorIntf;