/**
 * 数据插件的包装
 * Created by buns on 1/11/16.
 */

"use strict";
var util = require("util");

var Promise = require("bluebird");

var store = require("../store");
var error = require("../error");
var DIO = require("../model/po-dio");
var arrUtil = require("../utils/util-array");

function DAO(opt) {
    this.id = opt.alias || opt.engine;
}

/**
 * 执行查询前会进行参数的转换 此转换的目的是为了保证业务线中业务节点的sql配置中的sql执行能够正确执行
 * 1.先获取当前业务线当前业务点的所有有效sqlKey
 * 2.判断当前sqlKey是否是符合上述结果的
 * 3.将参数进行转换成DIO对象
 *
 * 参数形式:
 *  1.单条sql 单个参数键值对
 *  2.单条sql 参数键值对数组
 *  3.多条sql 单个参数键值对
 *  4.多条sql 多个参数键值对
 *
 *  string,object
 *  string,array[object]
 *  <string,object>
 *  <string,array[object]>
 *  array[string] object
 *  array[array]
 *
 * @returns {*}
 */
DAO.prototype.exec = function () {
    var daoInstance,
        allSqlKey,
        allSqlConfig,
        sqlConfig;

    var self = this;

    var arg = arguments;

    daoInstance = store.getPlug(self.id).instance;

    allSqlKey = store.getBusiLineConfig(self.lineKey).nodes[self.nodeKey].sqlKey;

    if (!allSqlKey) {
        throw error.errorBuild(error.eErrorType.NotFindSqlKeys, self.lineKey, self.nodeKey);
    }

    allSqlKey = allSqlKey.split(',');

    //获取所有sql配置
    allSqlConfig = store.getPlug(self.id).configs;

    var dios = [], i, l;

    if (arg.length === 1) {
        if (util.isArray(arg[0])) { //针对参数传入形式是: [[sqlKey,paramDic],[sqlKey,paramDic]]
            for (l = arg[0].length; l--;) {
                if (!arrUtil.contains(allSqlKey, arg[0][l][0])) {
                    throw error.errorBuild(error.eErrorType.NoAccessToSqlKey, self.lineKey, self.nodeKey, arg[0][l][0]);
                }
                sqlConfig = allSqlConfig[arg[0][l][0]];

                dios[l] = buildDIO(sqlConfig, arg[0][l][1], self);
            }
        } else {
            for (i in arg[0]) { //针对参数传入形式是: <sqlKey,paramDic> <sqlKey,paramDic[]>
                if (!arg[0].hasOwnProperty(i)) continue;

                if (!arrUtil.contains(allSqlKey, i)) {
                    throw error.errorBuild(error.eErrorType.NoAccessToSqlKey, self.lineKey, self.nodeKey, i);
                }
                sqlConfig = allSqlConfig[i];
                dios.push(buildDIO(sqlConfig, arg[0][i], self));
            }
        }
    } else if (arg.length === 2) {
        if (util.isArray(arg[0])) {//针对 sqlKey[] paramDic
            for (l = arg[0].length; l--;) {
                if (!arrUtil.contains(allSqlKey, arg[0][l])) {
                    throw error.errorBuild(error.eErrorType.NoAccessToSqlKey, self.lineKey, self.nodeKey, arg[0][l]);
                }
                sqlConfig = allSqlConfig[arg[0][l]];
                dios[l] = buildDIO(sqlConfig, arg[1], self);
            }
        } else {//针对 sqlKey,paramDic 和 sqlKey,paramDic[]

            if (!arrUtil.contains(allSqlKey, arg[0])) {
                throw error.errorBuild(error.eErrorType.NoAccessToSqlKey, self.lineKey, self.nodeKey, arg[0]);
            }

            sqlConfig = allSqlConfig[arg[0]];
            dios.push(buildDIO(sqlConfig, arg[1], self));
        }
    }

    return call(daoInstance, dios[0].type, dios);
};


/**
 * 构建数据操作交互对象
 * @param config
 * @param paramDic
 * @param dao
 * @returns {module.exports|exports}
 */
function buildDIO(config, paramDic, dao) {
    var dio = new DIO();

    dio.values = mapParamsDicToArr(paramDic, config.param);

    if (util.isString(dio.values)) {
        throw error.errorBuild(error.eErrorType.NotMatchSqlParams, dao.lineKey, dao.nodeKey, dio.values);
    }

    dio.sql = config.sql;
    dio.type = config.type;

    return dio;
}

/**
 * 将键值对对象转换成数组类型的数据结构
 *  转换过程会按照sql配置中的参数顺序进行匹配，如果在valueDic总没有找到对应的参数，则会返回该参数名称以表示改参数没有找到的异常
 * @param valueDic
 * @param paramSort
 * @returns {*}
 */
function mapParamsDicToArr(valueDic, paramSort) {

    var result = [], paramName;

    if (util.isString(paramSort) && paramSort.indexOf(',') > -1) {
        paramSort = paramSort.split(',');
    }

    var i, l;

    if (util.isArray(valueDic)) {
        if (valueDic.length > 1) {
            for (i = 0, l = valueDic.length; i < l; i++) {
                paramName = mapParamsDicToArr(valueDic[i], paramSort);
                if (util.isString(paramName)) {
                    result = null;
                    return paramName;
                }
                result.push(paramName);
            }
            return result;
        } else {
            valueDic = valueDic[0];
        }
    }

    for (i = 0, l = paramSort.length; i < l; i++) {
        paramName = paramSort[i].name;
        if (!valueDic.hasOwnProperty(paramName)) {
            result = null;
            return paramName;
        }
        result.push(valueDic[paramName]);
    }
    return result;
}

/**
 * 获取intf/dao中的实际数据操作
 *  该操作只提供三种接口:
 *      exec:执行新增/修改/删除的统一操作语句
 *      get:获取详情数据
 *      stream:流式的读取列表数据
 * @param dao
 * @param verb
 * @param dio
 * @returns {*}
 */
function call(dao, verb, dio) {
    return new Promise((function (_dao, _verb, _dio) {
        var result;
        switch (_verb) {
            case "detail":
                result = _dao.prototype.detail.call(_dao, _dio);
                break;
            case "insert":
            case "update":
            case "delete":
                result = _dao.prototype.exec.call(_dao, _dio);
                break;
            default:
                result = _dao.prototype.list.call(_dao, _dio);
                break;
        }
        return result;
    }(dao, verb, dio)));
}

module.exports = DAO;