"use strict";
var fs = require("fs");
var debug = require("debug")("stream-coding");

var util = require("util");
var fileUtil = require("./utils/util-file");

var DAO = require("./plugin/dao");
var RemoteMsg = require("./plugin/remoteMsg");
var ThirdIntf = require("./plugin/thirdIntf");

var data = {},
    busiLines = data.lines = {},
    busiNodeProcessors = data.nodes = {},
    plugs = data.plugs = {};

/**
 * 解析配置文件中的业务线参数
 * @param params
 * @param inParams
 * @param outParams
 */
function analysisLineParams(params, inParams, outParams) {
    var inP, outP;
    var i, l, param;

    if (!params) return;

    for (i in params) {
        if (!params.hasOwnProperty(i)) continue;
        param = params[i];
        inP = param.in;
        if (!inP) continue;

        inP = inP.split(',');
        l = inP.length;
        delete param.in;

        for (; l--;) {
            (inParams[inP[l]] || (inParams[inP[l]] = [])).push({
                key: i,
                maxLength: param.maxLength,
                opt: param.opt || false
            });
        }

        outP = param.out;
        if (!outP) continue;
        outP = outP.split(',');
        delete param.out;
        l = outP.length;
        for (; l--;) {
            (outParams[outP[l]] || (outParams[outP[l]] = [])).push({
                key: i,
                maxLength: param.maxLength,
                opt: param.opt || false
            });
        }
    }
}

var configAnalysis = {
    /**
     * 初始化业务流配置文件
     * @param config
     */
    busiLine: function analysisBusiLineConfig(config) {
        var contents = fileUtil.loadConfigFile(config);
        contents.forEach(function (item) {
            var line, node, it;
            for (line in item) {
                if (!item.hasOwnProperty(line))continue;
                busiLines[line] = it = item[line];
                for (node in it.nodes) {
                    if (!it.nodes.hasOwnProperty(node)) continue;
                    it.nodes[node].id = node;
                }

                it.inP = {};
                it.outP = {};
                analysisLineParams(it.params, it.inP, it.outP);
                delete it.params;
            }
            line = null;
        });
    },
    /**
     * 解析数据访问插件的配置
     * @param it
     */
    dao: function daoConfigAnalysis(it) {
        if (!it.alias) {
            throw new Error(' the "alias" it of stream-coding.plug-dao config must set value.');
        }

        it.instance = require(it.package || it.classPath);

        it.instance = new it.instance(it.ctorOption);

        var config = it.configs = {};

        if (it.configFiles) {
            fileUtil.loadConfigFile(it.configFiles).forEach(function (item) {
                for (var it in item) {
                    if (!item.hasOwnProperty(it)) continue;
                    config[it] = item[it];
                }
            });
        }

        var plugArr = plugs[it.alias];

        if (plugArr) {
            throw new Error(' the "alias" it of stream-coding.plug-dao config must set unique.');
        }
        plugs[it.alias] = it;

        it.plugInstance = new DAO(it);
    },
    /**
     * 解析第三方接口的插件配置
     * @param it
     */
    thirdIntf: function thirdIntfConfignalysis(it) {
        if (!it.alias) {
            throw new Error(' the "alias" it of stream-coding.plug-thirdIntf config must set value.');
        }

        var params = it.params = {};


        if (it.configFiles) {
            fileUtil.loadConfigFile(it.configFiles).forEach(function (item) {
                for (var it in item) {
                    if (!item.hasOwnProperty(it)) continue;
                    config[it] = item[it];
                }
            });
        }

        var plugArr = plugs[it.alias];

        if (plugArr) {
            throw new Error(' the "alias" it of stream-coding.plug-thirdIntf config must set unique.');
        }
        plugs[it.alias] = it;

        it.plugInstance = new ThirdIntf(it);
    },
    /**
     * 解析远程消息传输的插件配置
     * @param it
     */
    remoteMsg: function remoteMsgConfigAnalysis(it) {
        if (!it.alias) {
            throw new Error(' the "alias" it of stream-coding.plug-remoteMsg config must set value.');
        }

        it.instance = require(it.package || it.classPath);

        it.instance = new it.instance(it.ctorOption);

        var config = it.configs = {};

        if (it.configFiles) {
            fileUtil.loadConfigFile(it.configFiles).forEach(function (item) {
                for (var it in item) {
                    if (!item.hasOwnProperty(it)) continue;
                    config[it] = item[it];
                }
            });
        }

        var plugArr = plugs[it.alias];

        if (plugArr) {
            throw new Error(' the "alias" it of stream-coding.plug-remoteMsg config must set unique.');
        }
        plugs[it.alias] = it;

        it.plugInstance = new RemoteMsg(it);
    }
};

/**
 * 解析业务流配置,并将其缓存在store存储单元中
 */
exports.store = function store(globalConfig) {

    debug("开始状态stream-coding业务流程配置信息...");

    var item = globalConfig.appName;
    if (!item) {
        throw new Error(' the "appName" item of stream-coding config must set value.');
    }
    data.appName = item;

    item = globalConfig.busiProcessorRoot;
    if (!item) {
        throw new Error(' the "busiProcessorRoot" item of stream-coding config must set value.');
    }
    var lastChar = item.substring(item.length - 1);
    if (lastChar !== '/')
        item += "/";

    data.busiProcessorRoot = item;

    item = globalConfig.busiLineConfigPath;
    if (!item) {
        throw new Error(' the "busiLineConfigPath" item of stream-coding config must set value.');
    }

    data.busiLineConfigPath = item;

    configAnalysis.busiLine(item);

    debug("业务流程线配置信息装载完毕!");

    debug("开始装载插件配置信息...");

    [
        "dao",
        "remoteMsg",
        "thirdIntf"
    ].forEach(function (item) {
            var config = globalConfig["plug_" + item];
            if (config) {
                if (util.isArray(config)) {
                    config.forEach(configAnalysis[item]);
                } else {
                    configAnalysis[item](config);
                }
            }
        });

    debug("插件配置信息装载完成!");
};

/**
 * 获取处理器
 * @param path
 * @param lineKey
 * @param nodeKey
 * @returns {*}
 */
exports.getNodeProcessor = function (path, lineKey, nodeKey) {
    var key = data.busiProcessorRoot + path;

    var Node = busiNodeProcessors[key];

    if (!Node) {
        try {
            Node = require(key);

            Node = busiNodeProcessors[key] = new Node();

        } catch (e) {
            return null;
        }
    }
    var it;
    for (var plug in plugs) {
        if (!plugs.hasOwnProperty(plug)) continue;
        it = plugs[plug].plugInstance;
        it.lineKey = lineKey;
        it.nodeKey = nodeKey;
        Node[it.id] = it;
    }
    return Node;
};

/**
 * 获取业务流配置
 * @param lineKey
 * @returns {*}
 */
exports.getBusiLineConfig = function (lineKey) {
    if (!lineKey) return;
    return busiLines[lineKey];
};

/**
 * 获取插件配置
 * @param alias 插件别名
 * @returns {*}
 */
exports.getPlug = function (alias) {
    return plugs[alias];
};